#!/usr/bin/env node
"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const command_line_args_1 = __importDefault(require("command-line-args"));
const command_line_usage_1 = __importDefault(require("command-line-usage"));
const controller_1 = require("./controller");
const function_1 = require("./function");
const test_1 = require("./test");
const model_1 = require("./model");
const repository_1 = require("./repository");
const sdkgen_1 = require("./sdkgen");
const fs_1 = require("fs");
const helpers_1 = require("./helpers");
var Database;
(function (Database) {
    Database[Database["typeorm"] = 0] = "typeorm";
    Database[Database["knex"] = 1] = "knex";
    Database[Database["puresql"] = 2] = "puresql";
})(Database || (Database = {}));
const optionDefinitions = [
    { alias: "c", name: "component", type: String },
    { alias: "f", multiple: true, name: "functions", type: String },
    { alias: "m", multiple: true, name: "model", type: String },
    { alias: "d", defaultValue: Database.typeorm, name: "database", type: String },
    { alias: "r", name: "repository", type: Boolean },
    { alias: "t", name: "test", type: Boolean },
    { alias: "h", name: "help", type: Boolean },
];
const options = command_line_args_1.default(optionDefinitions, {
    stopAtFirstUnknown: true,
});
if (options.help) {
    const sections = [
        {
            content: "Generates codes to make your life easier.",
            header: "Cubos CLI",
        },
        {
            header: "Options",
            optionList: [
                {
                    alias: "c",
                    description: "Component you want to create.",
                    name: "component",
                    type: String,
                },
                {
                    alias: "f",
                    description: "Provide functions you want to create.",
                    name: "functions",
                    type: String,
                },
                {
                    alias: "m",
                    description: "Model you want to create.",
                    name: "model",
                    type: String,
                },
                {
                    alias: "d",
                    description: "SQL Database that you want to use.",
                    name: "database",
                    type: String,
                },
                {
                    alias: "r",
                    description: "Provide if you want create repository file.",
                    name: "repository",
                    type: Boolean,
                },
                {
                    alias: "t",
                    description: "Provide if you want create tests.",
                    name: "test",
                    type: Boolean,
                },
                {
                    alias: "h",
                    description: "Print the guide to use this CLI.",
                    name: "help",
                    type: Boolean,
                },
            ],
        },
        {
            content: [
                {
                    desc: "1. Create a controller.",
                    example: "$ npx cubos-cli -c ted",
                },
                {
                    desc: "2. Create a controller with tests.",
                    example: "$ npx cubos-cli -c ted -t",
                },
                {
                    desc: "3. Create a controller with the function addSegure and tests.",
                    example: "$ npx cubos-cli -c segure -f addSegure#name:string,info:string -t",
                },
                {
                    desc: "4. Create a controller with functions and create a model",
                    example: "npx cubos-cli -c segure -f addSegure#name:string,info:string updateSegure#id:string,name:string,info:string -m name:string info:string",
                },
            ],
            header: "Examples",
        },
        {
            content: "Project home: {underline https://github.com/leleofg/cubos-cli}",
        },
    ];
    console.log(command_line_usage_1.default(sections));
    throw new Error("");
}
if (options.database !== Database.typeorm) {
    throw new Error("For now works only with typeorm.");
}
if (!options.component) {
    throw new Error("You need to provide the component.\n\ncubos-cli -h for more information");
}
try {
    if (controller_1.generateController(options.component)) {
        function_1.generateFunctions(options.component, options.functions);
    }
    sdkgen_1.generateSdkgen(options.component, options.functions);
    if (options.model) {
        model_1.generateModel(options.component, options.model);
    }
    if (options.test) {
        test_1.generateTest(options.component, options.functions);
    }
    if (options.repository) {
        if (!options.model) {
            throw new Error("You need to provide model when use repository.");
        }
        repository_1.generateRepository(options.component);
    }
}
catch (error) {
    // Controller
    fs_1.unlinkSync(`src/controllers/${options.component.toLocaleLowerCase()}.ts`);
    helpers_1.removeLineExistsInFile("src/controllers/index.ts", options.component.toLocaleLowerCase());
    // Sdkgen
    if (fs_1.existsSync(`src/schemas/${options.component.toLocaleLowerCase()}.sdkgen`)) {
        fs_1.unlinkSync(`src/schemas/${options.component.toLocaleLowerCase()}.sdkgen`);
        helpers_1.removeLineExistsInFile("src/api.sdkgen", options.component.toLocaleLowerCase());
    }
    // Model
    if (fs_1.existsSync(`src/models/${helpers_1.firstWordToUppercase(options.component)}.ts`)) {
        fs_1.unlinkSync(`src/models/${helpers_1.firstWordToUppercase(options.component)}.ts`);
        helpers_1.removeLineExistsInFile("src/models/index.ts", helpers_1.firstWordToUppercase(options.component));
    }
    // Tests
    if (fs_1.existsSync(`tests/${options.component.toLocaleLowerCase()}.test.ts`)) {
        fs_1.unlinkSync(`tests/${options.component.toLocaleLowerCase()}.test.ts`);
    }
    // Repository
    if (fs_1.existsSync(`src/repositories/${helpers_1.firstWordToUppercase(options.component)}Repository.ts`)) {
        fs_1.unlinkSync(`src/repositories/${helpers_1.firstWordToUppercase(options.component)}Repository.ts`);
        helpers_1.removeLineExistsInFile("src/models/index.ts", `${helpers_1.firstWordToUppercase(options.component)}Repository`);
    }
    console.log(error.message);
    console.log("cubos-cli -h for more information");
}
